"""
Abstractions for writing entrypoints
"""

from __future__ import annotations
import functools
import logging
import runpy
import sys
from typing import Callable

import attrs
import toolz

from . import commands as commandslib, api


def dunder_main(globals_dct, command_data, logger=logging.getLogger()):
    """
    Call from ``__main__``
    """
    if globals_dct["__name__"] != "__main__":
        raise ImportError("module cannot be imported", globals_dct["__name__"])
    ch = logging.StreamHandler()
    ch.setLevel(logging.INFO)
    formatter = logging.Formatter("%(asctime)s:%(levelname)s:%(name)s:%(message)s")
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    commandslib.run_maybe_dry(
        parser=commandslib.set_parser(collected=command_data.collector.collect()),
        is_subcommand=globals_dct.get("IS_SUBCOMMAND", False),
        prefix=command_data.prefix,
        argv=sys.argv,
    )


def _noop(_ignored):  # pragma: no cover
    pass


@attrs.frozen
class EntryData:
    """
    Data for the entry point.
    """

    prefix: str
    collector: api.Collector
    register: Callable
    main_command: Callable[[], None]
    sub_command: Callable[[], None]

    @classmethod
    def create(cls, package_name, prefix=None):
        """
        Create a new instance from package_name and prefix
        """
        if prefix is None:
            prefix = package_name
        collector = api.Collector()
        register = commandslib.make_command_register(collector)
        main_command = toolz.compose(
            _noop,
            functools.partial(
                runpy.run_module,
                package_name,
                run_name="__main__",
            ),
        )
        sub_command = functools.partial(
            main_command, init_globals=dict(IS_SUBCOMMAND=True)
        )
        return cls(
            prefix=prefix,
            collector=collector,
            register=register,
            main_command=main_command,
            sub_command=sub_command,
        )



"""Test gather's API"""
import unittest

import gather
from gather import unique

from gather.tests import _helper

MAIN_COMMANDS = gather.Collector()

OTHER_COMMANDS = gather.Collector()


@MAIN_COMMANDS.register()
def main1(args):
    """Plugin registered with name of function"""
    return "main1", args


@MAIN_COMMANDS.register(name="weird_name")
def main2(args):
    """Plugin registered with explicit name"""
    return "main2", args


@MAIN_COMMANDS.register(name="bar")
@OTHER_COMMANDS.register(name="weird_name")
def main3(args):
    """Plugin registered for two collectors"""
    return "main3", args


@OTHER_COMMANDS.register(name="baz")
def main4(args):
    """Plugin registered for the other collector"""
    return "main4", args


@_helper.weird_decorator
def weird_function():
    """Plugin using a wrapper function to register"""


TRANSFORM_COMMANDS = gather.Collector()


@TRANSFORM_COMMANDS.register(transform=gather.Wrapper.glue(5))
def fooish():
    """Plugin registered with a transformation"""


COLLIDING_COMMANDS = gather.Collector()

NON_COLLIDING_COMMANDS = gather.Collector()


@NON_COLLIDING_COMMANDS.register(name="weird_name")
@COLLIDING_COMMANDS.register(name="weird_name")
def weird_name1():
    """One of several commands registered for same name"""


@COLLIDING_COMMANDS.register(name="weird_name")
def weird_name2():
    """One of several commands registered for same name"""


@COLLIDING_COMMANDS.register(name="weird_name")
def weird_name3():
    """One of several commands registered for same name"""


class CollectorTest(unittest.TestCase):

    """Tests for collecting plugins"""

    def test_collecting(self):
        """Collecting gives only the registered plugins for a given collector"""
        collected = unique(MAIN_COMMANDS.collect())
        self.assertIn("main1", collected)
        self.assertIs(collected["main1"], main1)
        self.assertNotIn("baz", collected)

    def test_non_collision(self):
        """Collecting with same name for different collectors does not collide"""
        main = unique(MAIN_COMMANDS.collect())
        other = unique(OTHER_COMMANDS.collect())
        self.assertIs(main["weird_name"], main2)
        self.assertIs(main["bar"], main3)
        self.assertIs(other["weird_name"], main3)

    def test_cross_module_collection(self):
        """Collection works when plugins are registered in a different module"""
        collected = unique(_helper.WEIRD_COMMANDS.collect())
        self.assertIn("weird_function", collected)

    def test_transform(self):
        """Collecting transformed plugins applies transform on collection"""
        collected = unique(TRANSFORM_COMMANDS.collect())
        self.assertIn("fooish", collected)
        res = collected.pop("fooish")
        self.assertIs(res.original, fooish)
        self.assertEqual(res.extra, 5)

    def test_multiple(self):
        """Without unique, it gets all the registered plugins for name"""
        collected = COLLIDING_COMMANDS.collect()
        weird_name = collected.pop("weird_name")
        self.assertEqual(collected, {})
        self.assertEqual(weird_name, set([weird_name1, weird_name2, weird_name3]))

    def test_multiple_unique_fails(self):
        """Without unique, it gets all the registered plugins for name"""
        with self.assertRaises(ValueError):
            unique(COLLIDING_COMMANDS.collect())



"""Gather: The Plugin Gatherer"""
import importlib.metadata

from gather.api import Collector, Wrapper, unique

__version__ = importlib.metadata.version(__name__)

__all__ = ["Collector", "Wrapper", "unique", "__version__"]



"""Gather -- Collect all your plugins

Gather allows a way to register plugins.
It features the ability to register the plugins from any module,
in any package, in any distribution.
A given module can register plugins of multiple types.

In order to have anything registered from a package,
it needs to declare that it supports :code:`gather`
in its package metadata.

For example,
with
:code:`pyproject.toml`:

.. code::

    [project.entry-points.gather]
    ignored = "<ROOT_PACKAGE>"


The :code:`ROOT_PACKAGE` should point to the Python name of the package:
i.e., what users are expected to :code:`import` at the top-level.

Note that while having special facilities to run functions as subcommands,
Gather can be used to collect anything.
"""
import collections
import importlib.metadata
import sys

import attr
import venusian


def _get_modules():
    for entry_point in importlib.metadata.entry_points(group="gather"):
        module = importlib.import_module(entry_point.value)
        yield module


@attr.s(frozen=True)
class Collector(object):

    """
    A plugin collector.

    A collector allows to *register* functions or classes by modules,
    and *collect*-ing them when they need to be used.
    """

    name = attr.ib(default=None)

    depth = attr.ib(default=1)

    def register(self, name=None, transform=lambda x: x):
        """
        Register a class or function

        Args:
            name (str): optional. Name to register the class or function as.
                        (default is name of object)
            transform (callable): optional. A one-argument function. Will be called,
                          and the return value used in collection.
                          Default is identity function

        This is meant to be used as a decoator:

        .. code::

            @COLLECTOR.register()
            def specific_subcommand(args):
                pass

            @COLLECTOR.register(name='another_specific_name')
            def main(args):
                pass
        """

        def callback(scanner, inner_name, objct):
            (
                """
            Venusian_ callback, called from scan

            .. _Venusian: http://docs.pylonsproject.org/projects/"""
                """venusian/en/latest/api.html#venusian.attach
            """
            )
            tag = getattr(scanner, "tag", None)
            if tag is not self:
                return
            if name is None:
                effective_name = inner_name
            else:
                effective_name = name
            objct = transform(objct)
            scanner.registry[effective_name].add(objct)

        def attach(func):
            """Attach callback to be called when object is scanned"""
            venusian.attach(func, callback, depth=self.depth)
            return func

        return attach

    def collect(self):
        """
        Collect all registered functions or classes.

        Returns a dictionary mapping names to registered elements.
        """

        def ignore_import_error(_unused):
            """
            Ignore ImportError during collection.

            Some modules raise import errors for various reasons,
            and should be just treated as missing.
            """
            if not issubclass(sys.exc_info()[0], ImportError):
                raise  # pragma: no cover

        registry = collections.defaultdict(set)
        scanner = venusian.Scanner(registry=registry, tag=self)
        for module in _get_modules():
            scanner.scan(module, onerror=ignore_import_error)
        return registry


def unique(mapping):
    """
    Transform map to sets to map to single items.

    Raises a :code:`ValueError` if any of the values is not an iterable
    with exactly one item.

    Args:
        mapping: A mapping of keys to Iterables of 1

    Returns:
        A mapping of keys to the single value
    """
    ret = {}
    for key, value_set in mapping.items():
        [value] = value_set
        ret[key] = value
    return ret


@attr.s(frozen=True)
class Wrapper(object):

    """Add extra data to an object"""

    original = attr.ib()

    extra = attr.ib()

    @classmethod
    def glue(cls, extra):
        """
        Glue extra data to an object

        Args:
            extra: what to add

        Returns:
            callable: function of one argument that returns a :code:`Wrapped`

        This method is useful mainly as the :code:`transform` parameter
        of a :code:`register` call.
        """

        def ret(original):
            """Return a :code:`Wrapper` with the original and extra"""
            return cls(original=original, extra=extra)

        return ret


__all__ = ["Collector", "unique", "Wrapper"]



"""Breakfast plugins"""
import sys

import gather

BREAKFAST = gather.Collector()


def breakfast(_args):
    """Collect breakfast plugins, make breakfast"""
    foods = [klass() for klass in BREAKFAST.collect().values()]
    for food in foods:
        food.prepare()
    for food in foods:
        food.eat()


@BREAKFAST.register()
class Eggs(object):

    """Eggs plugin for breakfast"""

    def prepare(self):
        """Prepare eggs by scrambling"""
        sys.stdout.write("Scrambling eggs\n")

    def eat(self):
        """Eat the eggs by devouring"""
        sys.stdout.write("Devouring eggs\n")


@BREAKFAST.register()
class Cereal(object):

    """Cereal plugin for breakfast"""

    def prepare(self):
        """Prepare cereal by mixing it with milk"""
        sys.stdout.write("Mixing cereal and milk\n")

    def eat(self):
        """Eat cereal with a spoon"""
        sys.stdout.write("Eating cereal with a spoon\n")


@BREAKFAST.register()
class OrangeJuice(object):

    """OJ plugin for breakfast"""

    def prepare(self):
        """Prepare juice by squeezing it"""
        sys.stdout.write("Squeezing orange juice\n")

    def eat(self):
        """Consume the juice by drinking it"""
        sys.stdout.write("Drinking orange juice\n")



"""Test the __init__.py module"""

import unittest
from hamcrest import assert_that, contains_string

from .. import __version__


class TestInit(unittest.TestCase):
    """Tests for the __init__.py module"""

    def test_version(self):
        """Version has a . in it"""
        assert_that(__version__, contains_string("."))



"""Test entrypoint"""
import io
import logging
import unittest
from unittest import mock

from hamcrest import assert_that, calling, contains_string, equal_to, raises

from .. import entry

ENTRY_DATA = entry.EntryData.create("test_dunder_main")


@ENTRY_DATA.register(name="fake")
def _fake(args):
    print("hello")


class DunderMainTest(unittest.TestCase):

    """Test dunder_main"""

    def test_failed_import(self):
        """
        Function fails if the name is not __main__
        """
        assert_that(
            calling(entry.dunder_main).with_args(
                globals_dct=dict(__name__="some_name"),
                command_data=None,
                logger=None,
            ),
            raises(ImportError),
        )

    def test_run_command(self):
        """
        The fake command is called when the command line specifies it
        """
        logger = logging.Logger("nonce")
        mock_output = mock.patch("sys.stdout", new=io.StringIO())
        self.addCleanup(mock_output.stop)
        fake_stdout = mock_output.start()
        mock_args = mock.patch("sys.argv", new=[])
        self.addCleanup(mock_args.stop)
        fake_args = mock_args.start()
        fake_args[:] = ["test", "fake"]
        entry.dunder_main(
            globals_dct=dict(__name__="__main__"),
            logger=logger,
            command_data=ENTRY_DATA,
        )
        assert_that(fake_stdout.getvalue(), contains_string("hello"))

    def test_with_prefix(self):
        """
        An explicit prefix overrides the default
        """
        ed = entry.EntryData.create("test_dunder_main", prefix="thing")
        assert_that(ed.prefix, equal_to("thing"))



"""Wrap a plugin registration in a function"""
import gather

WEIRD_COMMANDS = gather.Collector(depth=2)


def weird_decorator(func):
    """Register function into WEIRD_COMMANDS"""
    WEIRD_COMMANDS.register()(func)
    return func



"""Test command dispatch"""

import argparse
import contextlib
import io
import pathlib
import os
import tempfile
import textwrap
import subprocess
import sys
import unittest
from unittest import mock
from hamcrest import (
    assert_that,
    all_of,
    has_key,
    has_entry,
    not_,
    string_contains_in_order,
    contains_string,
    calling,
    raises,
)


import gather
from gather import commands
from gather.commands import add_argument

COMMANDS_COLLECTOR = gather.Collector()

REGISTER = commands.make_command_register(COMMANDS_COLLECTOR)


@REGISTER(
    add_argument("--value", default="default-value"),
    name="do-something",
)
def _do_something(*, args, env, run):
    print(args.__gather_name__)
    print(args.value)
    print(env["SHELL"])
    run([sys.executable, "-c", "print(2)"], check=True)


@REGISTER(
    add_argument("--no-dry-run", action="store_true"),
    name="do-something-else",
)
def _do_something_else(*, args, env, run):
    print(args.no_dry_run)
    print(env["SHELL"])
    run([sys.executable, "-c", "print(3)"], check=True)


MAYBE_DRY_COMMANDS_COLLECTOR = gather.Collector()

MAYBE_DRY_REGISTER = commands.make_command_register(MAYBE_DRY_COMMANDS_COLLECTOR)


@MAYBE_DRY_REGISTER(
    add_argument("--no-dry-run", action="store_true", default=False),
    add_argument("--output-dir", required=True),
    name="write-safely",
)
def _write_safely(args):
    output_dir = pathlib.Path(args.output_dir)
    safe = os.fspath(output_dir / "safe.txt")
    unsafe = os.fspath(output_dir / "unsafe.txt")
    code = textwrap.dedent(
        """\
    import pathlib
    import sys
    pathlib.Path(sys.argv[1]).write_text(str(1 + 1))
    """
    )
    args.run([sys.executable, "-c", code, unsafe])
    args.safe_run([sys.executable, "-c", code, safe])


class CommandTest(unittest.TestCase):

    """Test command dispatch"""

    def setUp(self):
        """Set up sys.stdio and a mock process runner"""
        mock_output = mock.patch("sys.stdout", new=io.StringIO())
        self.addCleanup(mock_output.stop)
        self.fake_stdout = mock_output.start()

        def mini_python(argv, *args, check=False, **kwargs):
            if argv[:2] != [sys.executable, "-c"]:
                raise ValueError("only minipython", argv)
            details = argv[2].removeprefix("python(").removesuffix(")")
            print(details)

        self.fake_run = mock.MagicMock(side_effect=mini_python)

    def test_simple_command(self):
        """Running a command dispatches to the registered function"""
        parser = commands.set_parser(collected=COMMANDS_COLLECTOR.collect())
        commands.run(
            parser=parser,
            argv=["command", "do-something"],
            env=dict(SHELL="some-shell"),
            sp_run=self.fake_run,
        )
        output = self.fake_stdout.getvalue()
        assert_that(
            output,
            string_contains_in_order(
                "do-something",
                "default-value",
                "some-shell",
                "2",
            ),
        )

    def test_custom_parser(self):
        """Custom help message is printed out"""
        parser = commands.set_parser(
            collected=COMMANDS_COLLECTOR.collect(),
            parser=argparse.ArgumentParser(
                description="this is a custom help message",
            ),
        )
        assert_that(
            calling(commands.run).with_args(
                parser=parser,
                argv=["command", "--help"],
            ),
            raises(SystemExit),
        )
        output = self.fake_stdout.getvalue()
        assert_that(
            output,
            contains_string("custom help message"),
        )


class CommandMaybeDryTest(unittest.TestCase):

    """Test run_maybe_dry"""

    def test_error(self):
        """Help message is printed out"""
        parser = commands.set_parser(collected=MAYBE_DRY_COMMANDS_COLLECTOR.collect())
        mock_output = mock.patch("sys.stdout", new=io.StringIO())
        self.addCleanup(mock_output.stop)
        fake_stdout = mock_output.start()
        assert_that(
            calling(commands.run_maybe_dry).with_args(
                parser=parser,
                argv=["command"],
                env={},
                sp_run=subprocess.run,
            ),
            raises(SystemExit),
        )
        output = fake_stdout.getvalue()
        assert_that(
            output,
            contains_string("usage"),
        )

    def test_with_dry(self):
        """Test running command in dry-run mode"""
        parser = commands.set_parser(collected=MAYBE_DRY_COMMANDS_COLLECTOR.collect())
        with contextlib.ExitStack() as stack:
            tmp_dir = pathlib.Path(stack.enter_context(tempfile.TemporaryDirectory()))
            commands.run_maybe_dry(
                parser=parser,
                argv=["command", "write-safely", "--output-dir", os.fspath(tmp_dir)],
                env={},
                sp_run=subprocess.run,
            )
            contents = {child.name: child.read_text() for child in tmp_dir.iterdir()}
        assert_that(
            contents,
            all_of(
                not_(has_key("unsafe.txt")),
                has_entry("safe.txt", "2"),
            ),
        )

    def test_with_no_dry(self):
        """Test running command in no dry-run mode"""
        parser = commands.set_parser(collected=MAYBE_DRY_COMMANDS_COLLECTOR.collect())
        with contextlib.ExitStack() as stack:
            tmp_dir = pathlib.Path(stack.enter_context(tempfile.TemporaryDirectory()))
            commands.run_maybe_dry(
                parser=parser,
                argv=[
                    "command",
                    "write-safely",
                    "--output-dir",
                    os.fspath(tmp_dir),
                    "--no-dry-run",
                ],
                env={},
                sp_run=subprocess.run,
            )
            contents = {child.name: child.read_text() for child in tmp_dir.iterdir()}
        assert_that(
            contents,
            all_of(
                has_entry("unsafe.txt", "2"),
                has_entry("safe.txt", "2"),
            ),
        )

    def test_with_dry_fail(self):
        """Test running command that fails"""
        parser = commands.set_parser(collected=MAYBE_DRY_COMMANDS_COLLECTOR.collect())
        with contextlib.ExitStack() as stack:
            tmp_dir = pathlib.Path(stack.enter_context(tempfile.TemporaryDirectory()))
            assert_that(
                calling(commands.run_maybe_dry).with_args(
                    parser=parser,
                    argv=[
                        "command",
                        "write-safely",
                        "--output-dir",
                        os.fspath(tmp_dir / "not-there"),
                    ],
                    env={},
                    sp_run=subprocess.run,
                ),
                raises(subprocess.CalledProcessError),
            )

    def test_with_subcommand(self):
        """Test running command as subcommand"""
        parser = commands.set_parser(collected=MAYBE_DRY_COMMANDS_COLLECTOR.collect())
        with contextlib.ExitStack() as stack:
            tmp_dir = pathlib.Path(stack.enter_context(tempfile.TemporaryDirectory()))
            commands.run_maybe_dry(
                parser=parser,
                argv=[
                    "write-safely",
                    "--output-dir",
                    os.fspath(tmp_dir),
                    "--no-dry-run",
                ],
                is_subcommand=True,
                env={},
                sp_run=subprocess.run,
            )
            contents = {child.name: child.read_text() for child in tmp_dir.iterdir()}
        assert_that(
            contents,
            all_of(
                has_entry("unsafe.txt", "2"),
                has_entry("safe.txt", "2"),
            ),
        )

    def test_with_prefixed_subcommand(self):
        """Test running command as subcommand"""
        parser = commands.set_parser(collected=MAYBE_DRY_COMMANDS_COLLECTOR.collect())
        with contextlib.ExitStack() as stack:
            tmp_dir = pathlib.Path(stack.enter_context(tempfile.TemporaryDirectory()))
            commands.run_maybe_dry(
                parser=parser,
                argv=[
                    "command-write-safely",
                    "--output-dir",
                    os.fspath(tmp_dir),
                    "--no-dry-run",
                ],
                is_subcommand=True,
                prefix="command",
                env={},
                sp_run=subprocess.run,
            )
            contents = {child.name: child.read_text() for child in tmp_dir.iterdir()}
        assert_that(
            contents,
            all_of(
                has_entry("unsafe.txt", "2"),
                has_entry("safe.txt", "2"),
            ),
        )



"""Registration and dispatch to sub-commands"""

from __future__ import annotations
import argparse
import functools
import logging
import os
import subprocess
import sys
from typing import Sequence, Any, Tuple

import attrs

from .api import Wrapper, unique

LOGGER = logging.getLogger(__name__)


@attrs.frozen
class _Argument:
    args: Sequence[Any]
    kwargs: Sequence[Tuple[str, Any]]


def add_argument(*args, **kwargs):
    """
    Add argument to a registered command.

    See :code:`argparse.ArgumentParser.add_argument`
    for a description of the argument semantics.
    """
    return _Argument(args, frozenset(kwargs.items()))


def _transform(*args):
    glue = Wrapper.glue(args)
    return glue


def make_command_register(collector):
    """
    Return a decorator that registers a command.

    Args:
        collector: Collector to add commands to

    Returns:
        Callable that expects positional add_argument arguments,
        and returns a decorator that registers the function
        to the collector.
    """

    def _register(*args, name=None):
        a_transform = _transform(*args)
        return collector.register(transform=a_transform, name=name)

    return _register


def set_parser(*, collected, parser=None):
    """
    Set (or create) a parser.

    The parser will dispatch to the functions collected.
    The parser will configure the argument parsing according to the
    function's :code:`add_argument` in the registration.

    Args:
        collected: Return value from :code:`Collector.collected`
        parser: an argument parser

    Returns:
        An argument parser
    """
    if parser is None:
        parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()
    commands = unique(collected)
    for name, details in commands.items():
        original = details.original
        args = details.extra
        a_subparser = subparsers.add_parser(name)
        a_subparser.set_defaults(
            __gather_name__=name,
            __gather_command__=original,
        )
        for arg_details in args:
            a_subparser.add_argument(*arg_details.args, **dict(arg_details.kwargs))
    return parser


def _make_safe_run(args):
    no_dry_run = getattr(args, "no_dry_run", False)
    orig_run = args.orig_run

    @functools.wraps(orig_run)
    def wrapped_run(cmdargs, **kwargs):
        real_kwargs = dict(text=True, check=True, capture_output=True)
        real_kwargs.update(kwargs)
        LOGGER.info("Running: %s", cmdargs)
        try:
            return orig_run(cmdargs, **real_kwargs)
        except subprocess.CalledProcessError as exc:
            exc.add_note(f"STDERR: {exc.stderr}")
            exc.add_note(f"STDOUT: {exc.stdout}")
            raise

    @functools.wraps(orig_run)
    def wrapped_dry_run(cmdargs, **kwargs):
        LOGGER.info("Running: %s", cmdargs)
        LOGGER.info("Dry run, skipping")

    unsafe_run = wrapped_run if no_dry_run else wrapped_dry_run
    args.run = unsafe_run
    args.safe_run = wrapped_run
    args.orig_run = orig_run


def run_maybe_dry(
    *,
    parser,
    argv=sys.argv,
    env=os.environ,
    sp_run=subprocess.run,
    is_subcommand=False,
    prefix=None,
):
    """
    Run commands that only take ``args``.

    This runs commands that take ``args``.
    Commands can assume that the following attributes
    exist:

    * ``run``: Run with logging, only if `--no-dry-run` is passed
    * ``safe_run``: Run with logging
    * ``orig_run``: Original function
    """

    def error(args):
        parser.print_help()
        raise SystemExit(1)

    argv = list(argv)
    if is_subcommand:
        argv[0:0] = [prefix or "base-command"]
        argv[1] = argv[1].rsplit("/", 1)[-1]
        if prefix is not None:
            argv[1] = argv[1].removeprefix(prefix + "-")

    args = parser.parse_args(argv[1:])
    args.orig_run = sp_run
    args.env = env
    _make_safe_run(args)
    try:
        command = args.__gather_command__
    except AttributeError:
        command = error
    return command(
        args=args,
    )


def run(*, parser, argv=sys.argv, env=os.environ, sp_run=subprocess.run):
    """
    Parse arguments and run the command.

    Pass non-default args in testing scenarios.

    Args:
        argv: sys.argv or something that looks like it
        env: os.environ or something that looks like it
        sp_run: subprocess.run or something that looks like it

    Returns:
        Return value from dispatched command
    """
    args = parser.parse_args(argv[1:])
    command = args.__gather_command__
    return command(
        args=args,
        env=env,
        run=sp_run,
    )



"""Example commands"""
import sys
import gather
from gather import commands
from gather.commands import add_argument

_COMMANDS_COLLECTOR = gather.Collector()
REGISTER = commands.make_command_register(_COMMANDS_COLLECTOR)


def get_parser():
    """Get parser dispatching to example commands"""
    return commands.set_parser(collected=_COMMANDS_COLLECTOR.collect())


@REGISTER(
    add_argument("--value"),
    name="do-something",
)
def _do_something(*, args, env, run):
    print(args.value)
    print(env["SHELL"])
    run([sys.executable, "-c", "print(1+1)"], check=True)


@REGISTER(
    add_argument("--no-dry-run", action="store_true"),
    name="do-something-else",
)
def _do_something_else(*, args, env, run):
    print(args.no_dry_run)
    print(env["SHELL"])
    run([sys.executable, "-c", "print(1+1)"], check=True)
